unit CobAlignedPanel;

interface

uses
  SysUtils, Classes, Controls, ExtCtrls, Graphics, Types;

type
  TCobAlignedPanel = class(TCustomPanel)
  private
    { Private declarations }
    FAlignedCaption: string;
    FMarginal: integer;
    //FCaptionAlign: string;
    procedure SetAlignedCaption(Value: string);
    procedure SetMarginal(Value: integer);
  protected
    { Protected declarations }
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    //property Caption;
    property Cursor;
    property DockSite;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HelpContext;
    property HelpKeyword;
    property HelpType;
    property Hint;
    property Locked;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property PopupMenu;
    property TabOrder;
    property TabStop;
    property Tag;
    property UseDockManager;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDblClick;
    property OnDockDrop;
    property OnDockOver;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEndDock;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
    //my properties
    property AlignedCaption: string read FAlignedCaption write SetAlignedCaption;
    property Marginal: integer read FMarginal write SetMarginal default 5;
  end;

procedure Register;

implementation

{$R CobAlignedPanel.res}

procedure Register;
begin
  RegisterComponents('Cobian', [TCobAlignedPanel]);
end;

{ TCobAlignedPanel }

constructor TCobAlignedPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Controlstyle := Controlstyle - [csSetCaption]; 
  Caption:= '';
  FMarginal:= 5;
end;

procedure TCobAlignedPanel.Paint();
var
  CHeight: integer;
  R: TRect;
begin
  inherited Paint;
  if Enabled then
    Canvas.Font.Color:= clWindowText else
    Canvas.Font.Color:= clGrayText;
  Canvas.Brush.Style:= bsClear;
  CHeight:= Canvas.TextHeight(FAlignedCaption);
  R.Left:= FMarginal;
  R.Top:= ((self.ClientHeight div 2) - (CHeight div 2));
  R.Right:= (self.ClientWidth - FMarginal - 3);
  R.Bottom:= R.Top + CHeight+ 2;

  Canvas.TextRect(R, FMarginal,((self.ClientHeight div 2) - (CHeight div 2)),FAlignedCaption);
end;

procedure TCobAlignedPanel.SetAlignedCaption(Value: string);
begin
  if FAlignedCaption <> Value then
    begin
      FAlignedCaption:= Value;
      Invalidate();
    end;
end;

procedure TCobAlignedPanel.SetMarginal(Value: integer);
begin
  if Value < 0 then
    Value:= 0;

  if FMarginal <> Value then
    begin
      FMarginal:= Value;
      Invalidate();
    end;
end;

end.
