{*******************************************************************************
***                                                                          ***
***                        2000-2004 by Luis Cobian                         ***
***                           All rights reserved                            ***
***                                                                          ***
***                           cobian@educ.umu.se                             ***
***                                                                          ***
***                                                                          ***
***                     A progress bar with color schemes                    ***
***                                                                          ***
***                                                                          ***
***                                                                          ***
***                                                                          ***
*******************************************************************************}


unit CobBar;

interface

uses
  SysUtils, Windows, Classes, Controls, Graphics, Messages;

type
  TColorScheme = (ccCustom, ccGreen, ccBlue, ccDeepBlue,
    ccRed, ccYellow, ccBlack, ccSilver, ccWhite,
    ccViolet, ccPink, ccOrange);
  TOrientation = (ccHorizontal, ccVertical);

  TCobBar = class(TGraphicControl)
  private
    FColorBorder: TColor;
    FColorFont: Tcolor;
    FColorFace: TColor;
    FColorBack: TColor;
    FColorScheme: TColorScheme;
    FPosition: Integer;
    FMin: Integer;
    FMax: Integer;
    FStep: Integer;
    FShowPosition: boolean;
    FShowPercent: boolean;
    FCaption: string;
    FOrientation: TOrientation;
    FBmp: TBitMap;
    procedure SetColorBorder(Value: TColor);
    procedure SetColorFace(Value: TColor);
    procedure SetColorBack(Value: TColor);
    procedure SetPosition(Value: Integer);
    procedure SetMin(Value: Integer);
    procedure SetMax(Value: Integer);
    procedure SetStep(Value: Integer);
    procedure SetColorScheme(Value: TColorScheme);
    procedure SetOrientation(Value: TOrientation);
    procedure SetCaption(Value: string);
    procedure SetShowPosition(Value: boolean);
    procedure SetShowPercent(Value: boolean);
    procedure SetFontColor(Value: TColor);
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure CMFontChanged(var msg: TMessage); message CM_FONTCHANGED;
    procedure CMEnabledChanged(var msg: TMessage); message CM_ENABLEDCHANGED;
    procedure Click;override;
    procedure DblClick;override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure StepIt();
  published
    { Old properties}
    property Left;
    property Top;
    property Width;
    property Height;
    property Cursor;
    property Hint;
    property HelpType;
    property HelpKeyword;
    property HelpContext;
    property Font;
    property ParentFont;
    property Enabled;
    property Visible;
    property ShowHint;
    property ParentShowHint;
    property DragCursor;
    property DragKind;
    property DragMode;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property PopupMenu;
    property OnClick;
    property OnDblClick;
    property Align;
    {My properties}
    property ColorBorder: TColor read FColorBorder write SetColorBorder default
      clBlack;
    property ColorFace: TColor read FColorFace write SetColorFace default
      clAqua;
    property ColorBack: TColor read FColorBack write SetColorBack default
      clGray;
    property Position: integer read FPosition write SetPosition default 0;
    property Min: integer read FMin write SetMin default 0;
    property Max: integer read FMax write SetMax default 100;
    property Step: integer read FStep write SetStep default 10;
    property ColorScheme: TColorScheme read FColorScheme write SetColorScheme
      default ccBlue;
    property Caption: string read FCaption write SetCaption;
    property ShowPosition: boolean read FShowPosition write SetShowPosition
      default false;
    property ShowPercent: boolean read FShowPercent write SetShowPercent default
      false;
    property Orientation: TOrientation read FOrientation write SetOrientation
      default ccHorizontal;
    property ColorFont: TColor read FColorFont write SetFontColor default
      clBlack;
  end;

procedure Register;

implementation

{$R CobBar.res}

procedure Register;
begin
  RegisterComponents('Cobian', [TCobBar]);
end;

{ TCobBar }

procedure TCobBar.Click;
begin
  if not Enabled then
    Exit;
  Paint;
  inherited;
end;

procedure TCobBar.CMEnabledChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

procedure TCobBar.CMFontChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

constructor TCobBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBmp := TBitmap.Create;
  Width := 200;
  Height := 21;
  FMin := 0;
  FMax := 100;
  FStep := 10;
  FPosition := 0;
  FOrientation := ccHorizontal;
  FColorBorder := $00005680;
  FColorFace := $00C6ECFF;
  FColorBack := clWhite;
  FColorFont := clBlack;
  ColorScheme := ccBlue;
  FCaption := '';
  FShowPosition := false;
  FShowPercent := false;
  ParentFont := true;
  ParentShowHint := true;
  FBmp.Canvas.Font := Font;
end;

procedure TCobBar.DblClick;
begin
  if not Enabled then
    Exit;
  Paint;
  inherited;
end;

destructor TCobBar.Destroy;
begin
  FBmp.Free;
  inherited Destroy;
end;

procedure TCobBar.Paint;
var
  Pos: integer;
  Text: string;
  Tw, Th: integer;
  CFace, CBack, CBorder, CFont: TColor;
  Rect: TRect;
begin
  FBmp.Canvas.Pen.Style := psSolid;
  FBmp.Canvas.Brush.Style := bsSolid;

  if Enabled then
  begin
    CFace := FColorFace;
    CBack := FColorBack;
    CBorder := FColorBorder;
    CFont := FColorFont;
  end
  else
  begin
    CFace := $00C8C8C8;
    CBack := $00EEEEEE;
    CBorder := clSilver;
    CFont := clGray;
  end;

  FBmp.Width := ClientWidth;
  FBmp.Height := ClientHeight;

  FBmp.Canvas.Pen.Color := CBorder;
  FBmp.Canvas.Pen.Width := 1;

  //Paint the border and background
  FBmp.Canvas.Brush.Color := CBack;
  FBmp.Canvas.Rectangle(0, 0, Width, Height);

  FBmp.Canvas.Brush.Color := CFace;

  case FOrientation of
    ccHorizontal:
      begin
        Pos := Round(FBmp.Width * (FPosition - FMin) / (FMax - FMin));
        FBmp.Canvas.Rectangle(0, 0, Pos, Height);
      end;
    ccVertical:
      begin
        // Rotate the image;
        Pos := Round(FBmp.Height * (FPosition - FMin) / (FMax - FMin));
        FBmp.Canvas.Rectangle(0, Height - Pos, Width, Height);
      end;
  end; //case

  Text := FCaption;

  if FShowPosition then
  begin
    if Text <> '' then
      Text := Text + ' ';
    Text := Text + IntToStr(Position);
    if FShowPercent then
      Text := Text + '%';
  end;

  if Trim(Text) <> '' then
  begin
    FBmp.Canvas.Font := Font;
    FBmp.Canvas.Font.Color := CFont;
    Th := FBmp.Canvas.TextHeight(Text);
    Tw := FBmp.Canvas.TextWidth(Text);
    FBmp.Canvas.Brush.Style := bsClear;
    Rect.Left:=(FBmp.Width div 2) - (Tw div 2)-1;
    Rect.Top:=(FBmp.Height div 2) - (Th div 2)-1;
    Rect.Right:=Rect.Left+ tw +1;
    Rect.Bottom:= Rect.Top+ th +1;
    DrawText(FBmp.Canvas.Handle,
             PChar(Text),
             length(Text),
             Rect,
             DT_CENTER or DT_VCENTER or DT_SINGLELINE or DT_NOCLIP);
    //if FOrientation= ccHorizontal then
    {FBmp.Canvas.TextOut((FBmp.Width div 2) - (Tw div 2),
      (FBmp.Height div 2) - (Th div 2),
      Text);  }
  end;

  Canvas.Draw(0, 0, FBmp);
end;

procedure TCobBar.SetCaption(Value: string);
begin
  if Value <> FCaption then
  begin
    FCaption := Value;
    Invalidate();
  end;
end;

procedure TCobBar.SetColorBack(Value: TColor);
begin
  if Value <> FColorBack then
  begin
    FColorBack := Value;
    SetColorScheme(ccCustom);
    Invalidate();
  end;
end;

procedure TCobBar.SetColorBorder(Value: TColor);
begin
  if Value <> FColorBorder then
  begin
    FColorBorder := Value;
    SetColorScheme(ccCustom);
    Invalidate();
  end;
end;

procedure TCobBar.SetColorFace(Value: TColor);
begin
  if Value <> FColorFace then
  begin
    FColorFace := Value;
    SetColorScheme(ccCustom);
    Invalidate();
  end;
end;

procedure TCobBar.SetColorScheme(Value: TColorScheme);
begin
  if Value <> FColorScheme then
  begin
    case Value of
      ccBlack:
        begin
          FColorBack := $00ABABAB;
          FColorBorder := clSilver;
          FColorFace := $00585858;
          FColorFont := clWhite;
        end;
      ccBlue:
        begin
          FColorBack := $00FAECE0;
          FColorBorder := clGray;
          FColorFace := $00FFB928;
          FColorFont := clBlack;
        end;
      ccDeepBlue:
        begin
          FColorBack := $00FFD0D0;
          FColorBorder := clGray;
          FColorFace := $00FF4848;
          FColorFont := clBlack;
        end;
      ccGreen:
        begin
          FColorBack := $00DDF9E8;
          FColorBorder := $00005680;
          FColorFace := $0061DE8F;
          FColorFont := clBlack;
        end;
      ccOrange:
        begin
          FColorBack := $00ACC7FF;
          FColorBorder := clNavy;
          FColorFace := $000080FF;
          FColorFont := clBlack;
        end;
      ccPink:
        begin
          FColorBack := $00FCC1FF;
          FColorBorder := clGray;
          FColorFace := $00FF77FF;
          FColorFont := clBlack;
        end;
      ccRed:
        begin
          FColorBack := $00B3B3FF;
          FColorBorder := clMaroon;
          FColorFace := $004040FF;
          FColorFont := clBlack;
        end;
      ccSilver:
        begin
          FColorBack := $00E9E9E9;
          FColorBorder := clGray;
          FColorFace := $00C9C1C0;
          FColorFont := clBlack;
        end;
      ccWhite:
        begin
          FColorBack := clWhite;
          FColorBorder := clSilver;
          FColorFace := $00F2F2F2;
          FColorFont := clBlack;
        end;
      ccViolet:
        begin
          FColorBack := $00FFDFEF;
          FColorBorder := clTeal;
          FColorFace := $00FF8AC5;
          FColorFont := clBlack;
        end;
      ccYellow:
        begin
          FColorBack := $00D9FFFF;
          FColorBorder := clOlive;
          FColorFace := $0082FFFF;
          FColorFont := clBlack;
        end;
      ccCustom:
        begin
          //Do nothing
        end;
    end; //case
    FColorScheme := Value;
    Invalidate();
  end;
end;

procedure TCobBar.SetFontColor(Value: TColor);
begin
  if Value <> FColorFont then
  begin
    FColorFont := Value;
    SetColorScheme(ccCustom);
    Invalidate();
  end;
end;

procedure TCobBar.SetMax(Value: Integer);
begin
  if Value = FMin then
    Exit;

  if Value < FPosition then
    FPosition := Value;

  if Value <> 100 then
    SetShowPercent(false);

  if Value <> FMax then
  begin
    FMax := Value;
    Paint();
  end;
end;

procedure TCobBar.SetMin(Value: Integer);
begin
  if Value = FMax then
    Exit;

  if Value > FPosition then
    FPosition := Value;

  if Value <> 0 then
    SetShowPercent(false);

  if Value <> FMin then
  begin
    FMin := Value;
    Paint();
  end;
end;

procedure TCobBar.SetOrientation(Value: TOrientation);
begin
  if Value <> FOrientation then
  begin
    FOrientation := Value;
    Invalidate();
  end;
end;

procedure TCobBar.SetPosition(Value: Integer);
begin
  if Value > FMax then
    Value := FMax;

  if Value < FMin then
    Value := FMin;

  if Value <> FPosition then
  begin
    FPosition := Value;
    Paint();
  end;
end;

procedure TCobBar.SetShowPercent(Value: boolean);
begin
  if (Min <> 0) or (Max <> 100) then
    Exit;
  if Value <> FShowPercent then
  begin
    if Value then
      SetShowPosition(true);
    FShowPercent := Value;
    Invalidate();
  end;
end;

procedure TCobBar.SetShowPosition(Value: boolean);
begin
  if Value <> FShowPosition then
  begin
    if not Value then
      SetShowPercent(false);
    FShowPosition := Value;
    Invalidate();
  end;
end;

procedure TCobBar.SetStep(Value: Integer);
begin
  if Value <> FStep then
  begin
    FStep := Value;
    Paint();
    //Invalidate();
  end;
end;

procedure TCobBar.StepIt();
begin
  SetPosition(FPosition + FStep);
end;

end.

