{*******************************************************************************
***                                                                          ***
***                        2000-2004 by Luis Cobian                         ***
***                           All rights reserved                            ***
***                                                                          ***
***                           cobian@educ.umu.se                             ***
***                                                                          ***
***                                                                          ***
***           A progress bar with color schemes (UNICODE version)            ***
***                                                                          ***
***                                                                          ***
***                                                                          ***
***                                                                          ***
*******************************************************************************}


unit CobBarW;

interface

uses
  SysUtils, Windows, Classes, Controls, Graphics, Messages;

type
  TColorSchemeW = (ccwCustom, ccwGreen, ccwBlue, ccwBrown,
    ccwRed, ccwYellow, ccwBlack, ccwSilver, ccwWhite,
    ccwViolet, ccwPink, ccwOrange);
  TOrientationW = (ccwHorizontal, ccwVertical);

  TCobBarW = class(TGraphicControl)
  private
    FColorBorder: TColor;
    FColorFont: TColor;
    FColorFace: TColor;
    FColorBack: TColor;
    FColorScheme: TColorSchemeW;
    FPosition: Integer;
    FMin: Integer;
    FMax: Integer;
    FStep: Integer;
    FShowPosition: boolean;
    FShowPercent: boolean;
    FCaption: WideString;
    FOrientation: TOrientationW;
    FBmp: TBitMap;
    // FHint: WideString; To add for future versions
    procedure SetColorBorder(Value: TColor);
    procedure SetColorFace(Value: TColor);
    procedure SetColorBack(Value: TColor);
    procedure SetPosition(Value: Integer);
    procedure SetMin(Value: Integer);
    procedure SetMax(Value: Integer);
    procedure SetStep(Value: Integer);
    procedure SetColorScheme(Value: TColorSchemeW);
    procedure SetOrientation(Value: TOrientationW);
    procedure SetCaption(Value: WideString);
    procedure SetShowPosition(Value: boolean);
    procedure SetShowPercent(Value: boolean);
    procedure SetFontColor(Value: TColor);
    // procedure SetHint(Value: WideString);
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure CMFontChanged(var msg: TMessage); message CM_FONTCHANGED;
    procedure CMEnabledChanged(var msg: TMessage); message CM_ENABLEDCHANGED;
    procedure Click;override;
    procedure DblClick;override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure StepIt();
  published
    { Old properties}
    property Left;
    property Top;
    property Width;
    property Height;
    property Cursor;
    property HelpType;
    property HelpKeyword;
    property HelpContext;
    property Font;
    property ParentFont;
    property Enabled;
    property Visible;
    property ShowHint;
    property ParentShowHint;
    property DragCursor;
    property DragKind;
    property DragMode;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property PopupMenu;
    property OnClick;
    property OnDblClick;
    property Align;
    {My properties}
    property ColorBorder: TColor read FColorBorder write SetColorBorder default
      clBlack;
    property ColorFace: TColor read FColorFace write SetColorFace default
      clAqua;
    property ColorBack: TColor read FColorBack write SetColorBack default
      clGray;
    property Position: integer read FPosition write SetPosition default 0;
    property Min: integer read FMin write SetMin default 0;
    property Max: integer read FMax write SetMax default 100;
    property Step: integer read FStep write SetStep default 10;
    property ColorScheme: TColorSchemeW read FColorScheme write SetColorScheme
      default ccwBlue;
    property Caption: WideString read FCaption write SetCaption;
    property ShowPosition: boolean read FShowPosition write SetShowPosition
      default false;
    property ShowPercent: boolean read FShowPercent write SetShowPercent default
      false;
    property Orientation: TOrientationW read FOrientation write SetOrientation
      default ccwHorizontal;
    property ColorFont: TColor read FColorFont write SetFontColor default
      clBlack;
    // property Hint: WideString read FHint write SetHint;
  end;

procedure Register;

implementation

uses
  Types;

{$R CobBarW.res}

procedure Register;
begin
  RegisterComponents('Cobian', [TCobBarW]);
end;

{ TCobBarW }

procedure TCobBarW.Click;
begin
  if not Enabled then
    Exit;
  Paint;
  inherited;
end;

procedure TCobBarW.CMEnabledChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

procedure TCobBarW.CMFontChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

constructor TCobBarW.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBmp := TBitmap.Create;
  Width := 200;
  Height := 21;
  FMin := 0;
  FMax := 100;
  FStep := 10;
  FPosition := 0;
  FOrientation := ccwHorizontal;
  FColorBorder := $00005680;
  FColorFace := $00C6ECFF;
  FColorBack := clWhite;
  FColorFont := clBlack;
  ColorScheme := ccwBlue;
  FCaption := '';
  // FHint:= '';
  FShowPosition := false;
  FShowPercent := false;
  ParentFont := true;
  ParentShowHint := true;
  FBmp.Canvas.Font := Font;
end;

procedure TCobBarW.DblClick;
begin
  if not Enabled then
    Exit;
  Paint;
  inherited;
end;

destructor TCobBarW.Destroy;
begin
  FBmp.Free;
  inherited Destroy;
end;

procedure TCobBarW.Paint;
var
  Pos: integer;
  Text: WideString;
  Tw, Th: integer;
  CFace, CBack, CBorder, CFont: TColor;
  Rect: TRect;
  Metrics: tagSIZE;
begin
  FBmp.Canvas.Pen.Style := psSolid;
  FBmp.Canvas.Brush.Style := bsSolid;

  if Enabled then
  begin
    CFace := FColorFace;
    CBack := FColorBack;
    CBorder := FColorBorder;
    CFont := FColorFont;
  end
  else
  begin
    CFace := $00C8C8C8;
    CBack := $00EEEEEE;
    CBorder := clSilver;
    CFont := clGray;
  end;

  FBmp.Width := ClientWidth;
  FBmp.Height := ClientHeight;

  FBmp.Canvas.Pen.Color := CBorder;
  FBmp.Canvas.Pen.Width := 1;

  //Paint the border and background
  FBmp.Canvas.Brush.Color := CBack;
  FBmp.Canvas.Rectangle(0, 0, Width, Height);

  FBmp.Canvas.Brush.Color := CFace;

  case FOrientation of
    ccwHorizontal:
      begin
        Pos := Round(FBmp.Width * (FPosition - FMin) / (FMax - FMin));
        FBmp.Canvas.Rectangle(0, 0, Pos, Height);
      end;
    ccwVertical:
      begin
        // Rotate the image;
        Pos := Round(FBmp.Height * (FPosition - FMin) / (FMax - FMin));
        FBmp.Canvas.Rectangle(0, Height - Pos, Width, Height);
      end;
  end; //case

  Text := FCaption;

  if FShowPosition then
  begin
    if Text <> '' then
      Text := Text + ' ';
    Text := Text + WideString(IntToStr(Position));
    if FShowPercent then
      Text := Text + '%';
  end;

  if Trim(Text) <> '' then
  begin
    FBmp.Canvas.Font := Font;
    FBmp.Canvas.Font.Color := CFont;
    GetTextExtentPoint32W(FBmp.Canvas.Handle,
                      PWideChar(Text),
                      Length(Text), Metrics);

    Th := Metrics.cy;
    Tw := Metrics.cx;
    FBmp.Canvas.Brush.Style := bsClear;
    Rect.Left:=(FBmp.Width div 2) - (Tw div 2)-1;
    Rect.Top:=(FBmp.Height div 2) - (Th div 2)-1;
    Rect.Right:=Rect.Left+ tw +1;
    Rect.Bottom:= Rect.Top+ th +1;

    DrawTextW(FBmp.Canvas.Handle,
             PWideChar(Text),
             length(Text),
             Rect,
             DT_CENTER or DT_VCENTER or DT_SINGLELINE or DT_NOCLIP);
    //if FOrientation= ccHorizontal then
    {FBmp.Canvas.TextOut((FBmp.Width div 2) - (Tw div 2),
      (FBmp.Height div 2) - (Th div 2),
      Text);  }
  end;

  Canvas.Draw(0, 0, FBmp);
end;

procedure TCobBarW.SetCaption(Value: WideString);
begin
  if Value <> FCaption then
  begin
    FCaption := Value;
    Invalidate();
  end;
end;

procedure TCobBarW.SetColorBack(Value: TColor);
begin
  if Value <> FColorBack then
  begin
    FColorBack := Value;
    SetColorScheme(ccwCustom);
    Invalidate();
  end;
end;

procedure TCobBarW.SetColorBorder(Value: TColor);
begin
  if Value <> FColorBorder then
  begin
    FColorBorder := Value;
    SetColorScheme(ccwCustom);
    Invalidate();
  end;
end;

procedure TCobBarW.SetColorFace(Value: TColor);
begin
  if Value <> FColorFace then
  begin
    FColorFace := Value;
    SetColorScheme(ccwCustom);
    Invalidate();
  end;
end;

procedure TCobBarW.SetColorScheme(Value: TColorSchemeW);
begin
  if Value <> FColorScheme then
  begin
    case Value of
      ccwBlack:
        begin
          FColorBack := $00ABABAB;
          FColorBorder := clSilver;
          FColorFace := $00585858;
          FColorFont := clWhite;
        end;
      ccwBlue:
        begin
          FColorBack := $00FAECE0;
          FColorBorder := clGray;
          FColorFace := $00FFB928;  
          FColorFont := clBlack;
        end;
      ccwBrown:
        begin
          FColorBack := $00E0E0EF;
          FColorBorder := clMaroon;
          FColorFace := $00BFBFDF;
          FColorFont := clBlack;
        end;
      ccwGreen:
        begin
          FColorBack := $00DDF9E8;
          FColorBorder := $00005680;
          FColorFace := $0061DE8F;
          FColorFont := clBlack;
        end;
      ccwOrange:
        begin
          FColorBack := $00ACC7FF;
          FColorBorder := clNavy;
          FColorFace := $000080FF;
          FColorFont := clBlack;
        end;
      ccwPink:
        begin
          FColorBack := $00FCC1FF;
          FColorBorder := clGray;
          FColorFace := $00FF77FF;
          FColorFont := clBlack;
        end;
      ccwRed:
        begin
          FColorBack := $00B3B3FF;
          FColorBorder := clMaroon;
          FColorFace := $004040FF;
          FColorFont := clBlack;
        end;
      ccwSilver:
        begin
          FColorBack := $00E9E9E9;
          FColorBorder := clGray;
          FColorFace := $00C9C1C0;
          FColorFont := clBlack;
        end;
      ccwWhite:
        begin
          FColorBack := clWhite;
          FColorBorder := clSilver;
          FColorFace := $00F2F2F2;
          FColorFont := clBlack;
        end;
      ccwViolet:
        begin
          FColorBack := $00FFDFEF;
          FColorBorder := clTeal;
          FColorFace := $00FF8AC5;
          FColorFont := clBlack;
        end;
      ccwYellow:
        begin
          FColorBack := $00D9FFFF;
          FColorBorder := clOlive;
          FColorFace := $0082FFFF;
          FColorFont := clBlack;
        end;
      ccwCustom:
        begin
          //Do nothing
        end;
    end; //case
    FColorScheme := Value;
    Invalidate();
  end;
end;

procedure TCobBarW.SetFontColor(Value: TColor);
begin
  if Value <> FColorFont then
  begin
    FColorFont := Value;
    SetColorScheme(ccwCustom);
    Invalidate();
  end;
end;

{procedure TCobBarW.SetHint(Value: WideString);
begin
  if Value <> FHint then
  begin
    FHint := Value;
    // Invalidate();   There is no need
  end;
end; }

procedure TCobBarW.SetMax(Value: Integer);
begin
  if Value = FMin then
    Exit;

  if Value < FPosition then
    FPosition := Value;

  if Value <> 100 then
    SetShowPercent(false);

  if Value <> FMax then
  begin
    FMax := Value;
    Paint();
  end;
end;

procedure TCobBarW.SetMin(Value: Integer);
begin
  if Value = FMax then
    Exit;

  if Value > FPosition then
    FPosition := Value;

  if Value <> 0 then
    SetShowPercent(false);

  if Value <> FMin then
  begin
    FMin := Value;
    Paint();
  end;
end;

procedure TCobBarW.SetOrientation(Value: TOrientationW);
begin
  if Value <> FOrientation then
  begin
    FOrientation := Value;
    Invalidate();
  end;
end;

procedure TCobBarW.SetPosition(Value: Integer);
begin
  if Value > FMax then
    Value := FMax;

  if Value < FMin then
    Value := FMin;

  if Value <> FPosition then
  begin
    FPosition := Value;
    Paint();
  end;
end;

procedure TCobBarW.SetShowPercent(Value: boolean);
begin
  if (Min <> 0) or (Max <> 100) then
    Exit;
  if Value <> FShowPercent then
  begin
    if Value then
      SetShowPosition(true);
    FShowPercent := Value;
    Invalidate();
  end;
end;

procedure TCobBarW.SetShowPosition(Value: boolean);
begin
  if Value <> FShowPosition then
  begin
    if not Value then
      SetShowPercent(false);
    FShowPosition := Value;
    Invalidate();
  end;
end;

procedure TCobBarW.SetStep(Value: Integer);
begin
  if Value <> FStep then
  begin
    FStep := Value;
    Paint();
    //Invalidate();
  end;
end;

procedure TCobBarW.StepIt();
begin
  SetPosition(FPosition + FStep);
end;

end.

