{*******************************************************************************
***                                                                          ***
***                        2000-2004 by Luis Cobian                         ***
***                           All rights reserved                            ***
***                                                                          ***
***                           cobian@educ.umu.se                             ***
***                                                                          ***
***                                                                          ***
***                  A button with color schemes and glyph support           ***
***                                                                          ***
***                                                                          ***
***                                                                          ***
***                                                                          ***
*******************************************************************************}


unit CobButton;

interface

uses
  SysUtils, Windows, Classes, Controls, Graphics, Messages, Forms;

type
  TColorScheme = (cbCustom, cbGreen, cbBlue, cbDeepBlue,
    cbRed, cbYellow, cbBlack, cbSilver, cbWhite,
    cbViolet, cbPink, cbOrange, cbButton);

  TStatus = (cbNormal, cbOver, cbDown);

  TCobButton = class(TCustomControl)
  private
    { Private declarations }
    FColorBorder: TColor;
    FColorText: Tcolor;
    FColorFace: TColor;
    FColorBorderOver: TColor;
    FColorTextOver: Tcolor;
    FColorFaceOver: TColor;
    FColorBorderDown: TColor;
    FColorTextDown: Tcolor;
    FColorFaceDown: TColor;
    FColorScheme: TColorScheme;
    FGlyph: TBitMap;
    FClicked: boolean;
    FTransparent: boolean;
    FStatus: TStatus;
    FSpacing: integer;
    //FCaption: string;
    FFocused: boolean;
    FDefault: boolean;
    FCancel: boolean;
    FModalResult: TModalResult;
   // procedure SetCaption(Value: string);
    procedure SetColorFace(Value: TColor);
    procedure SetColorText(Value: TColor);
    procedure SetColorBorder(Value: TColor);
    procedure SetColorFaceOver(Value: TColor);
    procedure SetColorTextOver(Value: TColor);
    procedure SetColorBorderOver(Value: TColor);
    procedure SetColorFaceDown(Value: TColor);
    procedure SetColorTextDown(Value: TColor);
    procedure SetColorBorderDown(Value: TColor);
    procedure SetColorScheme(Value: TColorScheme);
    procedure SetGlyph(Value: TBitMap);
    procedure SetTransparent(Value: boolean);
    //procedure SetStatus(Value: TStatus);
    procedure SetSpacing(Value: integer);
    procedure SetDefault(Value: boolean);
    procedure SetCancel(Value: boolean);
  protected
    { Protected declarations }
    procedure Paint;override;
    procedure CMFontChanged(var msg: TMessage); message CM_FONTCHANGED;
    procedure CMEnabledChanged(var msg: TMessage); message CM_ENABLEDCHANGED;
    procedure MouseEnter(var msg: TMessage); message CM_MOUSEENTER;
    procedure MouseLeave(var msg: TMessage); message CM_MOUSELEAVE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure WMSetFocus(var msg: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var msg: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMKeyUp(var msg: TWMKeyUp); message WM_KEYUP;
    procedure WMKeyDown(var msg: TWMKeyDown); message WM_KEYDOWN;
    procedure CMDialogKey(var msg: TCMDialogKey); message CM_DIALOGKEY;
    procedure CMDialogChar(var msg: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMTextChanged (var msg: TMessage); message CM_TEXTCHANGED;
    procedure SetModalResult(Value:TmodalResult);
    procedure Click; Override;
    procedure DblClick; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;
  published
    { Published declarations }
    property ColorFace: TColor read FColorFace write SetColorFace default
      clBtnFace;
    property ColorText: TColor read FColorText write SetColorText default
      clCaptionText;
    property ColorBorder: TColor read FColorBorder write SetColorBorder default
      cl3DDkShadow;
    property ColorFaceOver: TColor read FColorFaceOver write SetColorFaceOver
      default clBtnFace;
    property ColorTextOver: TColor read FColorTextOver write SetColorTextOver
      default clCaptionText;
    property ColorBorderOver: TColor read FColorBorderOver write
      SetColorBorderOver default cl3DDkShadow;
    property ColorFaceDown: TColor read FColorFaceDown write SetColorFaceDown
      default clBtnFace;
    property ColorTextDown: TColor read FColorTextDown write SetColorTextDown
      default clCaptionText;
    property ColorBorderDown: TColor read FColorBorderDown write
      SetColorBorderDown default cl3DDkShadow;
    property ColorScheme: TColorScheme read FColorScheme write SetColorScheme
      default cbBlue;
    property Glyph: TBitMap read FGlyph write SetGlyph;
    property Transparent: boolean read FTransparent write SetTransparent default
      true;
    property Cancel: boolean read FCancel write SetCancel default false;
    property Default: boolean read FDefault write SetDefault default false;
    property Spacing: integer read FSpacing write SetSpacing default 4;
    property ModalResult: TModalResult read FModalResult write SetModalResult default mrNone;
    property Left;
    property Top;
    property Width;
    property Height;
    property Cursor;
    property Hint;
    property Caption;//: string read FCaption write SetCaption;
    property HelpType;
    property HelpKeyword;
    property HelpContext;
    property Font;
    property ParentFont;
    property Enabled;
    property Visible;
    property ShowHint;
    property ParentShowHint;
    property DragCursor;
    property DragKind;
    property DragMode;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property PopupMenu;
    property OnClick;
    property OnDblClick;
    property Align;
    property Anchors;
    property TabStop;
  end;

procedure Register;

implementation

{$R CobButton.res}

procedure Register;
begin
  RegisterComponents('Cobian', [TCobButton]);
end;

{ TCobSpeedButton }

procedure TCobButton.Click;
begin
  if not Enabled then
    Exit;
  if Parent <> nil then
    if GetParentForm(self)<>nil then
      GetParentForm(self).ModalResult:= ModalResult;
  FStatus:= cbNormal;
  Paint;
  inherited;
end;

procedure TCobButton.CMDialogChar(var msg: TCMDialogChar);
begin
  with msg do
    if IsAccel(CharCode, Caption) and Enabled then
    begin
      Click;
      Result := 1;
      Exit;
    end;
  inherited;
end;

procedure TCobButton.CMDialogKey(var msg: TCMDialogKey);
begin
  with msg do
  begin
    if  ((CharCode = VK_RETURN) and FFocused) then
      begin
        Click;
        Result := 1;
        Exit;
      end;

    if  ((CharCode = VK_RETURN) and FDefault) then
      begin
        Click;
        Result := 1;
        Exit;
      end;

    if ((CharCode = VK_ESCAPE) and FCancel) then
      begin
        Click;
        Result := 1;
        Exit;
      end;
       {  (KeyDataToShiftState(KeyData) = []) and CanFocus then
    begin
      Click;
      Result := 1;
    end else if FDefault  then begin
      Click;
      Result := 1;
    end     }

      inherited;
  end;

end;

procedure TCobButton.CMEnabledChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

procedure TCobButton.CMFontChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

procedure TCobButton.CMTextChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

constructor TCobButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FGlyph := TBitmap.Create;
  Width := 75;
  Height := 25;
  FTransparent := true;
  FStatus := cbNormal;
  ParentFont := true;
  ParentShowHint := true;
  FSpacing := 4;
  FClicked := false;
  TabStop:=true;
  FFocused:=false;
  FCancel:=false;
  FDefault:=false;
  FModalResult:=mrNone;
  SetColorScheme(cbBlue);
end;

procedure TCobButton.DblClick;
begin
  if not Enabled then
    Exit;
  FStatus:= cbNormal;
  Paint;
  inherited;
end;

destructor TCobButton.Destroy;
begin
  FreeAndNil(FGlyph);
  inherited Destroy;
end;

procedure TCobButton.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button <> mbLeft then
    Exit;
  if not Enabled then
    Exit;
  inherited;
  FClicked := True;
  FStatus := cbDown;
  Paint;
end;

procedure TCobButton.MouseEnter(var msg: TMessage);
begin
  if csDesigning in ComponentState then
    Exit;
  if not Enabled then
    Exit;
  inherited;
  if FClicked then
    FStatus := cbDown
  else
    FStatus := cbOver;
  Paint;

end;

procedure TCobButton.MouseLeave(var msg: TMessage);
begin
  inherited;
  FStatus := cbNormal;
  Paint();
end;

procedure TCobButton.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited;
end;

procedure TCobButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;
  FClicked := false;
  if (x > 0) and (y > 0) and (x < width) and (y < height) then
    FStatus := cbOver
  else
    FStatus := cbNormal;
  Paint;
end;

procedure TCobButton.Paint;
var
  tx, ty, bx, by, dx: integer;
  CText, CBorder, CFace: TColor;
  Rect: TRect;
begin                  
    CBorder:=clBlack;
    CText:=clBlack;
    CFace:=clSilver;
    if Enabled then
      begin
        case FStatus of
          cbNormal:
            begin
              CText:=FColorText;
              CBorder:=FColorBorder;
              CFace:=FColorFace;
              Canvas.Pen.Style:=psSolid;
            end;
          cbOver:
            begin
              CText:=FColorTextOver;
              CBorder:=FColorBorderOver;
              CFace:=FColorFaceOver;
              Canvas.Pen.Style:=psSolid;
            end;
          cbDown:
            begin
              CText:=FColorTextDown;
              CBorder:=FColorBorderDown;
              CFace:=FColorFaceDown;
              Canvas.Pen.Style:=psDot;
            end;
          end;
      end else
      begin
        CFace := $00E9E9E9;
        CText := clGray;
        CBorder := clSilver;
      end;

    Canvas.Pen.Width:=1;
    Canvas.Pen.Color := CBorder;
    Canvas.Brush.Color:=CFace;
    Canvas.Rectangle(0, 0, ClientWidth, ClientHeight);

    Canvas.Font := Font;
    bx := 0;
    by := 0;
    tx := 0;
    ty :=0;

    if not Glyph.Empty then
    begin
      bx := FGlyph.Width;
      by := FGlyph.Height;
      if Trim(Caption)<>'' then
        bx:=bx+FSpacing;
    end;

    if Trim(Caption)<>'' then
      begin
        tx:=Canvas.TextWidth(Caption);
        ty:= Canvas.TextHeight(Caption);
      end;

    dx:= bx + tx;

    if not Glyph.Empty then
      begin
        if FTransparent then
          FGlyph.TransparentColor:= FGlyph.Canvas.Pixels[0, 0];
        FGlyph.Transparent:= FTransparent;
        Canvas.Draw((ClientWidth div 2) - (dx div 2),
                    (ClientHeight div 2) - (by div 2), FGlyph);
      end;

    if Trim(Caption)<>'' then
      begin
        Canvas.Font.Color:=CText;
        Canvas.Brush.Style:= bsClear;
        {Canvas.TextOut((ClientWidth div 2) - (dx div 2) + bx,
                        (ClientHeight div 2)-(ty div 2),
                        Caption); }
        Rect.Left:=(ClientWidth div 2) - (dx div 2) + bx-1;
        Rect.Top:=(ClientHeight div 2)-(ty div 2)-1;
        Rect.Right:= Rect.Left+tx+1;
        Rect.Bottom:=Rect.Top+ty+1;
        //Because
        DrawText(Canvas.Handle,
             PChar(Caption),
             length(Caption),
             Rect,
             DT_CENTER or DT_VCENTER or DT_SINGLELINE or DT_NOCLIP);

      end;

    if FFocused then
      begin
        Canvas.Pen.Width:=1;
        Canvas.Pen.Style:=psDot;
        if (FColorScheme=cbDeepBlue) or (FColorScheme=cbBlack) then
          Canvas.Pen.Color:=clWhite else
          Canvas.Pen.Color:=clBlack;
        Canvas.Brush.Style:=bsClear;
        Canvas.Rectangle(3,3,ClientWidth-3, ClientHeight-3);
      end;
end;

procedure TCobButton.SetCancel(Value: boolean);
begin
  if Value <> FCancel then
  begin
    FCancel := Value;
    Invalidate(); //don't need this now but I will add some shadow on the future
  end;
end;

{procedure TCobButton.SetCaption(Value: string);
begin
  if Value <> FCaption then
  begin
    FCaption := Value;
    Invalidate();
  end;
end;     }

procedure TCobButton.SetColorBorder(Value: TColor);
begin
  if Value <> FColorBorder then
  begin
    FColorBorder := Value;
    FColorScheme := cbCustom;
    Invalidate();
  end;
end;

procedure TCobButton.SetColorBorderDown(Value: TColor);
begin
  if Value <> FColorBorderDown then
  begin
    FColorBorderDown := Value;
    FColorScheme := cbCustom;
    Invalidate();
  end;
end;

procedure TCobButton.SetColorBorderOver(Value: TColor);
begin
  if Value <> FColorBorderOver then
  begin
    FColorBorderOver := Value;
    FColorScheme := cbCustom;
    Invalidate();
  end;
end;

procedure TCobButton.SetColorFace(Value: TColor);
begin
  if Value <> FColorFace then
  begin
    FColorFace := Value;
    FColorScheme := cbCustom;
    Invalidate();
  end;
end;

procedure TCobButton.SetColorFaceDown(Value: TColor);
begin
  if Value <> FColorFaceDown then
  begin
    FColorFaceDown := Value;
    FColorScheme := cbCustom;
    Invalidate();
  end;
end;

procedure TCobButton.SetColorFaceOver(Value: TColor);
begin
  if Value <> FColorFaceOver then
  begin
    FColorFaceOver := Value;
    FColorScheme := cbCustom;
    Invalidate();
  end;
end;

procedure TCobButton.SetColorScheme(Value: TColorScheme);
begin
  if Value <> FColorScheme then
  begin
    if Value <> cbCustom then
    begin
      case Value of
        cbBlue:
          begin
            FColorBorder := clBlue;
            FColorText := clBlack;
            FColorFace := $00FFB928;
            FColorBorderOver := clNavy;
            FColorTextOver := clBlue;
            FColorFaceOver := $00FEE8BA;
            FColorBorderDown := clNavy;
            FColorTextDown := clYellow;
            FColorFaceDown := $00FF8080;
          end;
        cbGreen:
          begin
            FColorBorder := clGreen;
            FColorText := clBlack;
            FColorFace := $00B0FD7D;
            FColorBorderOver := clOlive;
            FColorTextOver := clPurple;
            FColorFaceOver := $00D2FEB6;
            FColorBorderDown := clYellow;
            FColorTextDown := clWhite;
            FColorFaceDown := clOlive;
          end;
        cbDeepBlue:
          begin
            FColorBorder := clSkyBlue;
            FColorText := clWhite;
            FColorFace := $00804000;
            FColorBorderOver := clLime;
            FColorTextOver := clWhite;
            FColorFaceOver := $00FD3E43;
            FColorBorderDown := clTeal;
            FColorTextDown := clYellow;
            FColorFaceDown := clPurple;
          end;
        cbRed:
          begin
            FColorBorder := clNavy;
            FColorText := clBlack;
            FColorFace := $006A6AFF;
            FColorBorderOver := clGray;
            FColorTextOver := clBlack;
            FColorFaceOver := $00C6C6FF;
            FColorBorderDown := clPurple;
            FColorTextDown := clYellow;
            FColorFaceDown := $004046BF;
          end;
        cbYellow:
          begin
            FColorBorder := clNavy;
            FColorText := clBlack;
            FColorFace := $0080FFFF;
            FColorBorderOver := clMaroon;
            FColorTextOver := clBlack;
            FColorFaceOver := $00D7FFFF;
            FColorBorderDown := clMaroon;
            FColorTextDown := clPurple;
            FColorFaceDown := $0000EEEE;
          end;
        cbBlack:
          begin
            FColorBorder := clSilver;
            FColorText := clWhite;
            FColorFace := clBlack;
            FColorBorderOver := clBlue;
            FColorTextOver := clWhite;
            FColorFaceOver := $00484848;
            FColorBorderDown := clYellow;
            FColorTextDown := clYellow;
            FColorFaceDown := $00694341;
          end;
        cbSilver:
          begin
            FColorBorder := clMaroon;
            FColorText := clBlack;
            FColorFace := clSilver;
            FColorBorderOver := clBlack;
            FColorTextOver := clBlack;
            FColorFaceOver := $00E2E2E2;
            FColorBorderDown := clWhite;
            FColorTextDown := clYellow;
            FColorFaceDown := clGray;
          end;
        cbWhite:
          begin
            FColorBorder := clMaroon;
            FColorText := clBlack;
            FColorFace := clWhite;
            FColorBorderOver := clOlive;
            FColorTextOver := clBlack;
            FColorFaceOver := $00E8FFFF;
            FColorBorderDown := clWhite;
            FColorTextDown := clWhite;
            FColorFaceDown := $008C7B79;
          end;
        cbViolet:
          begin
            FColorBorder := clTeal;
            FColorText := clBlack;
            FColorFace := $00FE54E4;
            FColorBorderOver := clMaroon;
            FColorTextOver := clBlack;
            FColorFaceOver := $00FFB0F3;
            FColorBorderDown := clWhite;
            FColorTextDown := clWhite;
            FColorFaceDown := $00C800AA;
          end;
        cbPink:
          begin
            FColorBorder := clTeal;
            FColorText := clBlack;
            FColorFace := $00FF80FF;
            FColorBorderOver := clGreen;
            FColorTextOver := clBlack;
            FColorFaceOver := $00FFCCFF;
            FColorBorderDown := clWhite;
            FColorTextDown := clWhite;
            FColorFaceDown := $00F700EA;
          end;
        cbOrange:
          begin
            FColorBorder := clPurple;
            FColorText := clBlack;
            FColorFace := $003CA6EC;
            FColorBorderOver := clTeal;
            FColorTextOver := clBlack;
            FColorFaceOver := $009DBDFF;
            FColorBorderDown := clSkyBlue;
            FColorTextDown := clBlue;
            FColorFaceDown := $006295FF;
          end;
        cbButton:
          begin
            FColorBorder := clBtnShadow;
            FColorText := clBtnText;
            FColorFace := clBtnFace;
            FColorBorderOver := clBtnShadow;
            FColorTextOver := clBtnText;
            FColorFaceOver := clBtnShadow;
            FColorBorderDown := clCream;
            FColorTextDown := clWhite;
            FColorFaceDown := clGray;
          end;
      end; //case
    end;
    FColorScheme := Value;
    Invalidate();
  end;
end;

procedure TCobButton.SetColorText(Value: TColor);
begin
  if Value <> FColorText then
  begin
    FColorText := Value;
    FColorScheme := cbCustom;
    Invalidate();
  end;
end;

procedure TCobButton.SetColorTextDown(Value: TColor);
begin
  if Value <> FColorTextDown then
  begin
    FColorTextDown := Value;
    FColorScheme := cbCustom;
    Invalidate();
  end;
end;

procedure TCobButton.SetColorTextOver(Value: TColor);
begin
  if Value <> FColorTextOver then
  begin
    FColorTextOver := Value;
    FColorScheme := cbCustom;
    Invalidate();
  end;
end;

procedure TCobButton.SetDefault(Value: boolean);
begin
  if Value <> FDefault then
  begin
    FDefault := Value;
    Invalidate();  //don't need this now but I will add some shadow on the future
  end;
end;

procedure TCobButton.SetGlyph(Value: TBitMap);
begin
  FGlyph.Assign(Value);
  Invalidate;
end;

procedure TCobButton.SetModalResult(Value: TmodalResult);
begin
  if Value<>FModalResult then
    FModalResult:= Value;
end;

procedure TCobButton.SetSpacing(Value: integer);
begin
  if Value <> FSpacing then
  begin
    FSpacing := Value;
    Invalidate();
  end;
end;

{procedure TCobSpeedButton.SetStatus(Value: TStatus);
begin
  if Value <> FStatus then
  begin
    FStatus := Value;
    Invalidate();
  end;
end; }

procedure TCobButton.SetTransparent(Value: boolean);
begin
  if Value <> FTransparent then
  begin
    FTransparent := Value;
    Invalidate();
  end;
end;

procedure TCobButton.WMKeyDown(var msg: TWMKeyDown);
begin
  inherited;
  if msg.CharCode = VK_SPACE then
    FStatus:= cbDown;
  if msg.CharCode = VK_RETURN then
    Click;
  Paint;
end;

procedure TCobButton.WMKeyUp(var msg: TWMKeyUp);
begin
  inherited;
  if (msg.CharCode = VK_SPACE) then
    begin
      FStatus:= cbNormal;
      Paint;
      Click;
    end;
end;

procedure TCobButton.WMKillFocus(var msg: TWMKillFocus);
begin
  inherited;
  FFocused:= false;
  FStatus:= cbNormal;
  Paint;
end;

procedure TCobButton.WMSetFocus(var msg: TWMSetFocus);
begin
  inherited;
  FFocused:= true;
  Paint;
end;

end.

