{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
~~~~~~~~~~                                                            ~~~~~~~~~~
~~~~~~~~~~             Component to register system hot keys          ~~~~~~~~~~
~~~~~~~~~~                Copyright 2006 by Luis Cobian               ~~~~~~~~~~
~~~~~~~~~~                     cobian@educ.umu.se                     ~~~~~~~~~~
~~~~~~~~~~                    All rights reserved                     ~~~~~~~~~~
~~~~~~~~~~                                                            ~~~~~~~~~~
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}

unit CobHotKey;

interface

uses
  SysUtils, Classes, Windows, Messages;

type
  TCobShift  = (cobCtrl, cobAlt, cobShift, cobWindows);
  TCobModifiers = set of TCobShift;

  TCobHotKey = class(TComponent)
  private
    { Private declarations }
    FRegistrationFailed: TNotifyEvent;
    FRegistrationDone: TNotifyEvent;
    FUnregistrationDone: TNotifyEvent;
    FUnregistrationFailed: TNotifyEvent;
    FHotKeyEvent: TNotifyEvent;
    FID: string;
    FHotKey: word;
    FChar: word;
    FModifiers: word;
    Handle: HWnd;
    FActive: boolean;
  protected
    { Protected declarations }
    procedure MessageHandler(var Msg: TMessage);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent);override;
    destructor Destroy(); override;
    // add a hot key
    procedure RegisterHotKey(const Ch: word; const Modifiers: TCobModifiers);
    procedure UnregisterHotKey();
  published
    { Published declarations }
    property OnRegistrationFailed: TNotifyEvent read FRegistrationFailed
            write FRegistrationFailed;
    property OnRegistrationDone: TNotifyEvent read FRegistrationDone
            write FRegistrationDone;
    property OnUnregistrationFailed: TNotifyEvent read FUnregistrationFailed
            write FUnregistrationFailed;
    property OnUnregistrationDone: TNotifyEvent read FUnregistrationDone
            write FUnregistrationDone;
    property OnHotKey: TNotifyEvent read FHotKeyEvent
            write FHotKeyEvent;
    property Active: boolean read FActive default false;
  end;

procedure Register;

implementation

{$R CobHotKey.res}
const
  ICON_SIZE = 24;
  PICONNAME = 'TCOBHOTKEY';

procedure Register;
begin
  RegisterComponents('Cobian', [TCobHotKey]);
end;

{ TCobHotKey }

constructor TCobHotKey.Create(AOwner: TComponent);
var
  GUID: TGUID;
begin
  inherited Create(AOwner);
  FActive:= false;
  Handle:= AllocateHWnd(MessageHandler);
  CreateGUID(GUID);
  FID:= GUIDToString(GUID); // to create the atom
  FHotKey:= AddAtom(PChar(FID));
end;

destructor TCobHotKey.Destroy();
begin
  DeleteAtom(FHotKey);
  DeallocateHWnd(Handle);
  inherited Destroy();
end;

procedure TCobHotKey.MessageHandler(var Msg: TMessage);
begin
  with Msg do
    if (Msg = WM_HOTKEY) then
      begin
        if ((LParamLo = FModifiers) and (LParamHi = FChar)) then
          begin
            if Assigned(FHotKeyEvent) then
              FHotKeyEvent(self);
          end else
          Result := DefWindowProc(Handle, Msg, wParam, lParam);
      end else
      Result := DefWindowProc(Handle, Msg, wParam, lParam);
end;

procedure TCobHotKey.RegisterHotKey(const Ch: word;
  const Modifiers: TCobModifiers);
begin
  FChar:= Ch;
  FModifiers:= 0;

  if (cobShift in Modifiers) then
    FModifiers:= FModifiers or MOD_SHIFT;
  if (cobAlt in Modifiers) then
    FModifiers:= FModifiers or MOD_ALT;
  if (cobCtrl in Modifiers) then
    FModifiers:= FModifiers or MOD_CONTROL;
  if (cobWindows in Modifiers) then
    FModifiers:= FModifiers or MOD_WIN;

  if Windows.RegisterHotKey(self.Handle , FHotKey, FModifiers , FChar) then
    begin
      FActive:= true;
      if Assigned(FRegistrationDone) then
        FRegistrationDone(self);
    end else
    begin
      if Assigned(FRegistrationFailed) then
        FRegistrationFailed(self);
    end;
end;

procedure TCobHotKey.UnregisterHotKey();
begin
  if Windows.UnregisterHotKey(self.Handle, FHotKey) then
    begin
      FActive:= false;
      if Assigned(FUnregistrationDone) then
        FUnregistrationDone(self);
    end else
    begin
      if Assigned(FUnregistrationFailed) then
        FUnregistrationFailed(self);
    end;    
end;

end.
