{*******************************************************************************
***                                                                          ***
***                        2000-2004 by Luis Cobian                         ***
***                           All rights reserved                            ***
***                                                                          ***
***                           cobian@educ.umu.se                             ***
***                                                                          ***
***                                                                          ***
***                         A panel with color schemes                       ***
***                                                                          ***
***                                                                          ***
***                                                                          ***
***                                                                          ***
*******************************************************************************}


unit CobPanel;

interface

uses
  SysUtils, Windows, Classes, Controls, Graphics, ExtCtrls, Messages;

type
  TColorScheme = (cpCustom, cpGreen, cpBlue, cpDeepBlue,
    cpRed, cpYellow, cpBlack, cpSilver, cpWhite,
    cpViolet, cpPink, cpOrange, cpPanel);
  TCaptionAlign = (cpCenter, cpTop, cpBottom, cpLeft, cpRight,
    cpLeftTop, cpLeftBottom, cpRightTop, cpRightButtom);
  TCobPanel = class(TCustomPanel)
  private
    { Private declarations }
    FColorCaption: TColor;
    FColorFace: TColor;
    FColorBorder: TColor;
    FColorScheme: TColorScheme;
    //FFlat: boolean;
    FCaptionAlign: TCaptionAlign;
    procedure SetColorCaption(Value: TColor);
    procedure SetColorFace(Value: TColor);
    procedure SetColorBorder(Value: TColor);
    procedure SetColorScheme(Value: TColorScheme);
    //procedure SetFlat(Value: boolean);
    procedure SetCaptionAlign(Value: TCaptionAlign);
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure Click;override;
    procedure DblClick;override;
    procedure CMFontChanged(var msg: TMessage); message CM_FONTCHANGED;
    procedure CMEnabledChanged(var msg: TMessage); message CM_ENABLEDCHANGED;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property Align;
    property Alignment;
    property Anchors;
    //property AutoSize;
    //property BevelInner;
    //property BevelOuter;
    //property BevelWidth;
    property BorderStyle;
    property Caption;
    property Cursor;
    property DockSite;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HelpContext;
    property HelpKeyword;
    property HelpType;
    property Hint;
    property Locked;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property PopupMenu;
    property TabOrder;
    property TabStop;
    property Tag;
    property UseDockManager;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDblClick;
    property OnDockDrop;
    property OnDockOver;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEndDock;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
    //my properties
    property ColorCaption: TColor read FColorCaption write SetColorCaption
      default clCaptionText;
    property ColorFace: TColor read FColorFace write SetColorFace default
      clBtnFace;
    property ColorBorder: TColor read FColorBorder write SetColorBorder default
      clBlack;
    property ColorScheme: TColorScheme read FColorScheme write SetColorScheme
      default cpBlue;
    //property Flat: boolean read FFlat write SetFlat default true;
    property CaptionAlign: TCaptionAlign read FCaptionAlign write SetCaptionAlign
      default cpCenter;
  end;

procedure Register;

implementation

{$R CobPanel.res}

procedure Register;
begin
  RegisterComponents('Cobian', [TCobPanel]);
end;

{ TCobPanel }

constructor TCobPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  Width := 185;
  Height := 41;
  FCaptionAlign := cpCenter;
  //FFlat:= true;
  Caption := '';
  ParentFont := true;
  ParentShowHint := true;
  SetColorScheme(cpBlue);
end;

procedure TCobPanel.Paint();
var
  CFace: TColor;
  CText: TColor;
  CBorder: TColor;
  Dx, Dy, x, y: integer;
  rect: Trect;
begin
  if Enabled then
  begin
    CFace := FColorFace;
    CText := FColorCaption;
    CBorder := FColorBorder;
  end
  else
  begin
    CFace := $00E9E9E9;
    CText := clGray;
    CBorder := clSilver;
  end;

  Canvas.Brush.Color := CFace;
  Canvas.Pen.Color := CBorder;
  Canvas.Pen.Width := 1;
  {if not FFlat then
  begin
    Canvas.Rectangle(2, 2, ClientWidth - 2, ClientHeight - 2);
    Canvas.Pen.Color := clBtnHighlight;
    Canvas.Pen.Width := 2;
    Canvas.MoveTo(0, 0);
    Canvas.LineTo(ClientWidth - 1, 0);
    Canvas.MoveTo(0, 0);
    Canvas.LineTo(0, ClientHeight - 1);
    Canvas.Pen.Color := clBtnShadow;
    Canvas.MoveTo(ClientWidth - 1, 0);
    Canvas.LineTo(ClientWidth - 1, ClientHeight-1);
    Canvas.MoveTo(0, ClientHeight - 1);
    Canvas.LineTo(ClientWidth - 1, ClientHeight - 1);
  end else   }
  Canvas.Rectangle(0, 0, ClientWidth, ClientHeight);

  if Trim(Caption) = '' then
    Exit;

  x := 0;
  y := 0;
  Canvas.Brush.Style := bsClear;
  Canvas.Pen.Color := CText;
  Canvas.Font := Font;
  Canvas.Font.Color := CText;
  Dx := Canvas.TextWidth(Caption);
  Dy := Canvas.TextHeight(Caption);
  case FCaptionAlign of
    cpCenter:
      begin
        x := (ClientWidth div 2) - (Dx div 2);
        y := (ClientHeight div 2) - (Dy div 2);
      end;
    cpTop:
      begin
        x := (ClientWidth div 2) - (Dx div 2);
        y := 5;
      end;
    cpBottom:
      begin
        x := (ClientWidth div 2) - (Dx div 2);
        y := ClientHeight - Dy - 5;
      end;
    cpLeft:
      begin
        x := 5;
        y := (ClientHeight div 2) - (Dy div 2);
      end;
    cpRight:
      begin
        x := ClientWidth - Dx - 5;
        y := (ClientHeight div 2) - (Dy div 2);
      end;
    cpLeftTop:
      begin
        x := 5;
        y := 5;
      end;
    cpLeftBottom:
      begin
        x := 5;
        y := ClientHeight - Dy - 5;
      end;
    cpRightTop:
      begin
        x := ClientWidth - Dx - 5;
        y := 5;
      end;
    cpRightButtom:
      begin
        x := ClientWidth - Dx - 5;
        y := ClientHeight - Dy - 5;
      end;
  end; //case
  Rect.Left:=x-1;
  Rect.Top:= y-1;
  Rect.Right:=Rect.Left+ Dx+ 1;
  Rect.Bottom:= Rect.Top + Dy +1;
  //Canvas.TextOut(x, y, Caption);
  DrawText(Canvas.Handle,
             PChar(Caption),
             length(Caption),
             Rect,
             DT_CENTER or DT_VCENTER or DT_SINGLELINE or DT_NOCLIP);
end;

procedure TCobPanel.SetCaptionAlign(Value: TCaptionAlign);
begin
  if Value <> FCaptionAlign then
  begin
    FCaptionAlign := Value;
    Invalidate();
  end;
end;

procedure TCobPanel.SetColorBorder(Value: TColor);
begin
  if Value <> FColorBorder then
  begin
    FColorBorder := Value;
    SetColorScheme(cpCustom);
    Invalidate();
  end;
end;

procedure TCobPanel.SetColorFace(Value: TColor);
begin
  if Value <> FColorFace then
  begin
    FColorFace := Value;
    SetColorScheme(cpCustom);
    Invalidate();
  end;
end;

procedure TCobPanel.SetColorScheme(Value: TColorScheme);
begin
  if Value <> FColorScheme then
  begin
    if Value <> cpCustom then
    begin
      case Value of
        cpGreen:
          begin
            FColorCaption := clBlack;
            FColorFace := $00DBFEBC;
            FColorBorder := clGray;
          end;
        cpBlue:
          begin
            FColorCaption := clBlack;
            FColorFace := $00FFD2A6;
            FColorBorder := clPurple;
          end;
        cpDeepBlue:
          begin
            FColorCaption := clPurple;
            FColorFace := $00FCA88B;
            FColorBorder := clNavy;
          end;
        cpRed:
          begin
            FColorCaption := clNavy;
            FColorFace := $008C8FFB;
            FColorBorder := clRed;
          end;
        cpYellow:
          begin
            FColorCaption := clNavy;
            FColorFace := $00CAFFFF;
            FColorBorder := clOlive;
          end;
        cpBlack:
          begin
            FColorCaption := clWhite;
            FColorFace := clBlack;
            FColorBorder := clSkyBlue;
          end;
        cpSilver:
          begin
            FColorCaption := clBlack;
            FColorFace := clSilver;
            FColorBorder := clOlive;
          end;
        cpWhite:
          begin
            FColorCaption := clBlack;
            FColorFace := clWhite;
            FColorBorder := clOlive;
          end;
        cpViolet:
          begin
            FColorCaption := clBlack;
            FColorFace := $00FD8CE9;
            FColorBorder := clNavy;
          end;
        cpPink:
          begin
            FColorCaption := clBlack;
            FColorFace := $00EEB5FB;
            FColorBorder := clMaroon;
          end;
        cpOrange:
          begin
            FColorCaption := clBlack;
            FColorFace := $0091B5FF;
            FColorBorder := clPurple;
          end;
        cpPanel:
          begin
            FColorCaption := clBlack;
            FColorFace := clBtnFace;
            FColorBorder := cl3DDkShadow;
          end;
      end;
    end;
    FColorScheme := Value;
    Invalidate();
  end;
end;

procedure TCobPanel.SetColorCaption(Value: TColor);
begin
  if Value <> FColorCaption then
  begin
    FColorCaption := Value;
    SetColorScheme(cpCustom);
    Invalidate();
  end;
end;

{procedure TCobPanel.SetFlat(Value: boolean);
begin
  if Value <> FFlat then
  begin
    FFlat := Value;
    Invalidate();
  end;
end;  }

procedure TCobPanel.CMFontChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

procedure TCobPanel.CMEnabledChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

procedure TCobPanel.Click;
begin
  if not Enabled then
    Exit;
  //Paint;
  inherited;
end;

procedure TCobPanel.DblClick;
begin
  if not Enabled then
    Exit;
  //Paint;
  inherited;
end;

end.

