{*******************************************************************************
***                                                                          ***
***                        2000-2004 by Luis Cobian                         ***
***                           All rights reserved                            ***
***                                                                          ***
***                           cobian@educ.umu.se                             ***
***                                                                          ***
***                                                                          ***
***                      A speed button with color schemes                   ***
***                               for Cobian Backup 6                        ***
***                                                                          ***
***                                                                          ***
***                                                                          ***
*******************************************************************************}


unit CobSpeedButton;

interface

uses
  SysUtils,Windows, Classes, Controls, Graphics, Messages ;

type
  TColorScheme = (csCustom, csGreen, csBlue, csDeepBlue,
    csRed, csYellow, csBlack, csSilver, csWhite,
    csViolet, csPink, csOrange, csButton);

  TStatus = (csNormal, csOver, csDown);

  TCobSpeedButton = class(TGraphicControl)
  private
    { Private declarations }
    FColorBorder: TColor;
    FColorText: Tcolor;
    FColorFace: TColor;
    FColorBorderOver: TColor;
    FColorTextOver: Tcolor;
    FColorFaceOver: TColor;
    FColorBorderDown: TColor;
    FColorTextDown: Tcolor;
    FColorFaceDown: TColor;
    FColorScheme: TColorScheme;
    FGlyph: TBitMap;
    FClicked: boolean;
    FTransparent: boolean;
    FStatus: TStatus;
    FSpacing: integer;
    //FCaption: string;
    //procedure SetCaption(Value: string);
    procedure SetColorFace(Value: TColor);
    procedure SetColorText(Value: TColor);
    procedure SetColorBorder(Value: TColor);
    procedure SetColorFaceOver(Value: TColor);
    procedure SetColorTextOver(Value: TColor);
    procedure SetColorBorderOver(Value: TColor);
    procedure SetColorFaceDown(Value: TColor);
    procedure SetColorTextDown(Value: TColor);
    procedure SetColorBorderDown(Value: TColor);
    procedure SetColorScheme(Value: TColorScheme);
    procedure SetGlyph(Value: TBitMap);
    procedure SetTransparent(Value: boolean);
    //procedure SetStatus(Value: TStatus);
    procedure SetSpacing(Value: integer);
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure CMFontChanged(var msg: TMessage); message CM_FONTCHANGED;
    procedure CMEnabledChanged(var msg: TMessage); message CM_ENABLEDCHANGED;
    procedure CMTextChanged (var msg: TMessage); message CM_TEXTCHANGED;
    procedure MouseEnter(var msg: TMessage); message CM_MOUSEENTER;
    procedure MouseLeave(var msg: TMessage); message CM_MOUSELEAVE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure Click; override;
    procedure DblClick; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;
  published
    { Published declarations }
    property ColorFace: TColor read FColorFace write SetColorFace default
      clBtnFace;
    property ColorText: TColor read FColorText write SetColorText default
      clCaptionText;
    property ColorBorder: TColor read FColorBorder write SetColorBorder default
      cl3DDkShadow;
    property ColorFaceOver: TColor read FColorFaceOver write SetColorFaceOver
      default clBtnFace;
    property ColorTextOver: TColor read FColorTextOver write SetColorTextOver
      default clCaptionText;
    property ColorBorderOver: TColor read FColorBorderOver write
      SetColorBorderOver default cl3DDkShadow;
    property ColorFaceDown: TColor read FColorFaceDown write SetColorFaceDown
      default clBtnFace;
    property ColorTextDown: TColor read FColorTextDown write SetColorTextDown
      default clCaptionText;
    property ColorBorderDown: TColor read FColorBorderDown write
      SetColorBorderDown default cl3DDkShadow;
    property ColorScheme: TColorScheme read FColorScheme write SetColorScheme
      default csBlue;
    property Glyph: TBitMap read FGlyph write SetGlyph;
    property Transparent: boolean read FTransparent write SetTransparent default
      true;
    property Spacing: integer read FSpacing write SetSpacing default 4;
    property Left;
    property Top;
    property Width;
    property Height;
    property Cursor;
    property Hint;
    property Caption;//: string read FCaption write SetCaption;
    property HelpType;
    property HelpKeyword;
    property HelpContext;
    property Font;
    property ParentFont;
    property Enabled;
    property Visible;
    property ShowHint;
    property ParentShowHint;
    property DragCursor;
    property DragKind;
    property DragMode;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property PopupMenu;
    property OnClick;
    property OnDblClick;
    property Align;
  end;

procedure Register;

implementation

uses Types;

{$R CobSpeedButton.res}

procedure Register;
begin
  RegisterComponents('Cobian', [TCobSpeedButton]);
end;

{ TCobSpeedButton }


procedure TCobSpeedButton.Click;
begin
  if not Enabled then
    Exit;
  FStatus:= csNormal;
  Paint;
  inherited;
end;

procedure TCobSpeedButton.CMEnabledChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

procedure TCobSpeedButton.CMFontChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

procedure TCobSpeedButton.CMTextChanged(var msg: TMessage);
begin
  inherited;
  Invalidate();
end;

constructor TCobSpeedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FGlyph := TBitmap.Create;
  Width := 75;
  Height := 25;
  FTransparent := true;
  FStatus := csNormal;
  ParentFont := true;
  ParentShowHint := true;
  FSpacing := 4;
  FClicked := false;
  SetColorScheme(csBlue);
end;

procedure TCobSpeedButton.DblClick;
begin
  if not Enabled then
    Exit;
  FStatus:= csNormal;
  Paint;
  inherited;
end;

destructor TCobSpeedButton.Destroy;
begin
  FreeAndNil(FGlyph);
  inherited Destroy;
end;

procedure TCobSpeedButton.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button <> mbLeft then
    Exit;
  if not Enabled then
    Exit;
  inherited;
  FClicked := True;
  FStatus := csDown;
  Paint;
end;

procedure TCobSpeedButton.MouseEnter(var msg: TMessage);
begin
  if csDesigning in ComponentState then
    Exit;
  if not Enabled then
    Exit;
  inherited;
  if FClicked then
    FStatus := csDown
  else
    FStatus := csOver;
  Paint;

end;

procedure TCobSpeedButton.MouseLeave(var msg: TMessage);
begin
  inherited;
  FStatus := csNormal;
  Paint();
end;

procedure TCobSpeedButton.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited;
end;

procedure TCobSpeedButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;
  FClicked := false;
  if (x > 0) and (y > 0) and (x < width) and (y < height) then
    FStatus := csOver
  else
    FStatus := csNormal;
  Paint;
end;

procedure TCobSpeedButton.Paint;
var
  tx, ty, bx, by, dx: integer;
  CText, CBorder, CFace: TColor;
  Rect: TRect;
begin
  CBorder := clBlack;
  CText := clBlack;
  CFace := clSilver;
  if Enabled then
  begin
    case FStatus of
      csNormal:
        begin
          CText := FColorText;
          CBorder := FColorBorder;
          CFace := FColorFace;
          Canvas.Pen.Style := psSolid;
        end;
      csOver:
        begin
          CText := FColorTextOver;
          CBorder := FColorBorderOver;
          CFace := FColorFaceOver;
          Canvas.Pen.Style := psSolid;
        end;
      csDown:
        begin
          CText := FColorTextDown;
          CBorder := FColorBorderDown;
          CFace := FColorFaceDown;
          Canvas.Pen.Style := psDot;
        end;
    end;
  end
  else
  begin
    CFace := $00E9E9E9;
    CText := clGray;
    CBorder := clSilver;
  end;

  Canvas.Pen.Width := 1;
  Canvas.Pen.Color := CBorder;
  Canvas.Brush.Color := CFace;
  Canvas.Rectangle(0, 0, ClientWidth, ClientHeight);

  Canvas.Font := Font;
  bx := 0;
  by := 0;
  tx := 0;
  ty := 0;

  if not Glyph.Empty then
  begin
    bx := FGlyph.Width;
    by := FGlyph.Height;
    if Trim(Caption) <> '' then
      bx := bx + FSpacing;
  end;

  if Trim(Caption) <> '' then
  begin
    tx := Canvas.TextWidth(Caption);
    ty := Canvas.TextHeight(Caption);
  end;

  dx := bx + tx;

  if not Glyph.Empty then
  begin
    if FTransparent then
      FGlyph.TransparentColor := FGlyph.Canvas.Pixels[0, 0];
    FGlyph.Transparent := FTransparent;
    Canvas.Draw((ClientWidth div 2) - (dx div 2),
      (ClientHeight div 2) - (by div 2), FGlyph);
  end;

  if Trim(Caption) <> '' then
  begin
    Canvas.Font.Color := CText;
    Canvas.Brush.Style := bsClear;
    {Canvas.TextOut((ClientWidth div 2) - (dx div 2) + bx,
      (ClientHeight div 2) - (ty div 2),
      Caption); }
    Rect.Left:=(ClientWidth div 2) - (dx div 2) + bx-1;
    Rect.Top:=(ClientHeight div 2) - (ty div 2) -1;
    Rect.Right:= Rect.Left+tx+1;
    Rect.Bottom:=Rect.Top+ty+1;
    DrawText(Canvas.Handle,
             PChar(Caption),
             length(Caption),
             Rect,
             DT_CENTER or DT_VCENTER or DT_SINGLELINE or DT_NOCLIP);
  end;
end;

{procedure TCobSpeedButton.SetCaption(Value: string);
begin
  if Value <> FCaption then
  begin
    FCaption := Value;
    Invalidate();
  end;
end;  }

procedure TCobSpeedButton.SetColorBorder(Value: TColor);
begin
  if Value <> FColorBorder then
  begin
    FColorBorder := Value;
    FColorScheme := csCustom;
    Invalidate();
  end;
end;

procedure TCobSpeedButton.SetColorBorderDown(Value: TColor);
begin
  if Value <> FColorBorderDown then
  begin
    FColorBorderDown := Value;
    FColorScheme := csCustom;
    Invalidate();
  end;
end;

procedure TCobSpeedButton.SetColorBorderOver(Value: TColor);
begin
  if Value <> FColorBorderOver then
  begin
    FColorBorderOver := Value;
    FColorScheme := csCustom;
    Invalidate();
  end;
end;

procedure TCobSpeedButton.SetColorFace(Value: TColor);
begin
  if Value <> FColorFace then
  begin
    FColorFace := Value;
    FColorScheme := csCustom;
    Invalidate();
  end;
end;

procedure TCobSpeedButton.SetColorFaceDown(Value: TColor);
begin
  if Value <> FColorFaceDown then
  begin
    FColorFaceDown := Value;
    FColorScheme := csCustom;
    Invalidate();
  end;
end;

procedure TCobSpeedButton.SetColorFaceOver(Value: TColor);
begin
  if Value <> FColorFaceOver then
  begin
    FColorFaceOver := Value;
    FColorScheme := csCustom;
    Invalidate();
  end;
end;

procedure TCobSpeedButton.SetColorScheme(Value: TColorScheme);
begin
  if Value <> FColorScheme then
  begin
    if Value <> csCustom then
    begin
      case Value of
        csBlue:
          begin
            FColorBorder := clBlue;
            FColorText := clBlack;
            FColorFace := $00FFB928;
            FColorBorderOver := clNavy;
            FColorTextOver := clBlue;
            FColorFaceOver := $00FEE8BA;
            FColorBorderDown := clNavy;
            FColorTextDown := clYellow;
            FColorFaceDown := $00FF8080;
          end;
        csGreen:
          begin
            FColorBorder := clGreen;
            FColorText := clBlack;
            FColorFace := $00B0FD7D;
            FColorBorderOver := clOlive;
            FColorTextOver := clPurple;
            FColorFaceOver := $00D2FEB6;
            FColorBorderDown := clYellow;
            FColorTextDown := clWhite;
            FColorFaceDown := clOlive;
          end;
        csDeepBlue:
          begin
            FColorBorder := clSkyBlue;
            FColorText := clWhite;
            FColorFace := $00804000;
            FColorBorderOver := clLime;
            FColorTextOver := clWhite;
            FColorFaceOver := $00FD3E43;
            FColorBorderDown := clTeal;
            FColorTextDown := clYellow;
            FColorFaceDown := clPurple;
          end;
        csRed:
          begin
            FColorBorder := clNavy;
            FColorText := clBlack;
            FColorFace := $006A6AFF;
            FColorBorderOver := clGray;
            FColorTextOver := clBlack;
            FColorFaceOver := $00C6C6FF;
            FColorBorderDown := clPurple;
            FColorTextDown := clYellow;
            FColorFaceDown := $004046BF;
          end;
        csYellow:
          begin
            FColorBorder := clNavy;
            FColorText := clBlack;
            FColorFace := $0080FFFF;
            FColorBorderOver := clMaroon;
            FColorTextOver := clBlack;
            FColorFaceOver := $00D7FFFF;
            FColorBorderDown := clMaroon;
            FColorTextDown := clPurple;
            FColorFaceDown := $0000EEEE;
          end;
        csBlack:
          begin
            FColorBorder := clSilver;
            FColorText := clWhite;
            FColorFace := clBlack;
            FColorBorderOver := clBlue;
            FColorTextOver := clWhite;
            FColorFaceOver := $00484848;
            FColorBorderDown := clYellow;
            FColorTextDown := clYellow;
            FColorFaceDown := $00694341;
          end;
        csSilver:
          begin
            FColorBorder := clMaroon;
            FColorText := clBlack;
            FColorFace := clSilver;
            FColorBorderOver := clBlack;
            FColorTextOver := clBlack;
            FColorFaceOver := $00E2E2E2;
            FColorBorderDown := clWhite;
            FColorTextDown := clYellow;
            FColorFaceDown := clGray;
          end;
        csWhite:
          begin
            FColorBorder := clMaroon;
            FColorText := clBlack;
            FColorFace := clWhite;
            FColorBorderOver := clOlive;
            FColorTextOver := clBlack;
            FColorFaceOver := $00E8FFFF;
            FColorBorderDown := clWhite;
            FColorTextDown := clWhite;
            FColorFaceDown := $008C7B79;
          end;
        csViolet:
          begin
            FColorBorder := clTeal;
            FColorText := clBlack;
            FColorFace := $00FE54E4;
            FColorBorderOver := clMaroon;
            FColorTextOver := clBlack;
            FColorFaceOver := $00FFB0F3;
            FColorBorderDown := clWhite;
            FColorTextDown := clWhite;
            FColorFaceDown := $00C800AA;
          end;
        csPink:
          begin
            FColorBorder := clTeal;
            FColorText := clBlack;
            FColorFace := $00FF80FF;
            FColorBorderOver := clGreen;
            FColorTextOver := clBlack;
            FColorFaceOver := $00FFCCFF;
            FColorBorderDown := clWhite;
            FColorTextDown := clWhite;
            FColorFaceDown := $00F700EA;
          end;
        csOrange:
          begin
            FColorBorder := clPurple;
            FColorText := clBlack;
            FColorFace := $003CA6EC;
            FColorBorderOver := clTeal;
            FColorTextOver := clBlack;
            FColorFaceOver := $009DBDFF;
            FColorBorderDown := clSkyBlue;
            FColorTextDown := clBlue;
            FColorFaceDown := $006295FF;
          end;
        csButton:
          begin
            FColorBorder := clBtnShadow;
            FColorText := clBtnText;
            FColorFace := clBtnFace;
            FColorBorderOver := clBtnShadow;
            FColorTextOver := clBtnText;
            FColorFaceOver := clBtnShadow;
            FColorBorderDown := clCream;
            FColorTextDown := clWhite;
            FColorFaceDown := clGray;
          end;
      end; //case
    end;
    FColorScheme := Value;
    Invalidate();
  end;
end;

procedure TCobSpeedButton.SetColorText(Value: TColor);
begin
  if Value <> FColorText then
  begin
    FColorText := Value;
    FColorScheme := csCustom;
    Invalidate();
  end;
end;

procedure TCobSpeedButton.SetColorTextDown(Value: TColor);
begin
  if Value <> FColorTextDown then
  begin
    FColorTextDown := Value;
    FColorScheme := csCustom;
    Invalidate();
  end;
end;

procedure TCobSpeedButton.SetColorTextOver(Value: TColor);
begin
  if Value <> FColorTextOver then
  begin
    FColorTextOver := Value;
    FColorScheme := csCustom;
    Invalidate();
  end;
end;

procedure TCobSpeedButton.SetGlyph(Value: TBitMap);
begin
  FGlyph.Assign(Value);
  Invalidate;
end;

procedure TCobSpeedButton.SetSpacing(Value: integer);
begin
  if Value <> FSpacing then
  begin
    FSpacing := Value;
    Invalidate();
  end;
end;

{procedure TCobSpeedButton.SetStatus(Value: TStatus);
begin
  if Value <> FStatus then
  begin
    FStatus := Value;
    Invalidate();
  end;
end; }

procedure TCobSpeedButton.SetTransparent(Value: boolean);
begin
  if Value <> FTransparent then
  begin
    FTransparent := Value;
    Invalidate();
  end;
end;

end.

